//
//  DataManager.m
//  MediServe
//
//  Created by Christopher Galindo on 3/28/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "DataManager.h"

static DataManager *theManager;

@implementation DataManager

@synthesize currentUser = _currentUser;
@synthesize patientsList = _patientsList;
@synthesize usersList = _usersList;
@synthesize patientNumber = _patientNumber;

- (id)init
{
    if (self = [super init])
    {
        [self loadPatientsList];
        [self loadUsersList];
        self.patientNumber = [self.patientsList count] + 1;
        //NSLog(@"Documents folder is %@", [self documentsDirectory]);
        //NSLog(@"Data file path is %@", [self dataFilePathForPatients]);
    }
    return self;
}

+ (DataManager *)sharedDataManager
{
    @synchronized(self)
    {
        if (theManager == nil)
            theManager = [[DataManager alloc] init];
    }
    return theManager;
}

- (NSString *)documentsDirectory
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docmentsDirectory = [paths objectAtIndex:0];
    return docmentsDirectory;
}

- (NSString *)dataFilePathForPatients
{
    return [[self documentsDirectory] stringByAppendingPathComponent:@"PatientsList"];
}

- (NSString *)dataFilePathForUsers
{
    return [[self documentsDirectory] stringByAppendingPathComponent:@"UsersList"];
}

- (void)loadPatientsList
{
    NSString *path = [self dataFilePathForPatients];
    if ([[NSFileManager defaultManager] fileExistsAtPath:path])
    {
        NSData *data = [[NSData alloc] initWithContentsOfFile:path];
        NSKeyedUnarchiver *unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:data];
        self.patientsList = [unarchiver decodeObjectForKey:@"PatientsList"];
        [unarchiver finishDecoding];
    }
    else
    {
        self.patientsList = [[NSMutableArray alloc] initWithCapacity:1];
        
        
        // This is to load demo versions with pre-installed patients
        /* NSString *filePath = [[NSBundle mainBundle] pathForResource:@"PatientsList" ofType:nil];
         NSData *data = [[NSData alloc] initWithContentsOfFile:filePath];
         NSKeyedUnarchiver *unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:data];
         self.patientsList = [unarchiver decodeObjectForKey:@"PatientsList"];
         [unarchiver finishDecoding]; */
    }
}

- (void)loadUsersList
{
    NSString *path = [self dataFilePathForUsers];
    if ([[NSFileManager defaultManager] fileExistsAtPath:path])
    {
        NSData *data = [[NSData alloc] initWithContentsOfFile:path];
        NSKeyedUnarchiver *unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:data];
        self.usersList = [unarchiver decodeObjectForKey:@"UsersList"];
        [unarchiver finishDecoding];
    }
    else
    {
        self.usersList = [[NSMutableArray alloc] initWithCapacity:1];
        User *userObject;
        
        userObject = [[User alloc] init];
        userObject.userID = @"Admin";
        userObject.userPassword = @"Admin";
        userObject.displayName = @"Administrator";
        [self.usersList addObject:userObject];
        
        userObject = [[User alloc] init];
        userObject.userID = @"mediserve";
        userObject.userPassword = @"mediserve";
        userObject.displayName = @"Serve, Medi";
        [self.usersList addObject:userObject];
        
        userObject = [[User alloc] init];
        userObject.userID = @"asu";
        userObject.userPassword = @"asu";
        userObject.displayName = @"Sundevil, Sparky";
        [self.usersList addObject:userObject];
        
        userObject = [[User alloc] init];
        userObject.userID = @"doc";
        userObject.userPassword = @"doc";
        userObject.displayName = @"Galindo, Chris";
        [self.usersList addObject:userObject];
    }
}

- (void)savePatientsList
{
    NSMutableData *data = [[NSMutableData alloc] init];
    NSKeyedArchiver *archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
    [archiver encodeObject:self.patientsList forKey:@"PatientsList"];
    [archiver finishEncoding];
    [data writeToFile:[self dataFilePathForPatients] atomically:YES];
}

- (void)saveUsersList
{
    NSMutableData *data = [[NSMutableData alloc] init];
    NSKeyedArchiver *archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
    [archiver encodeObject:self.usersList forKey:@"UsersList"];
    [archiver finishEncoding];
    [data writeToFile:[self dataFilePathForUsers] atomically:YES];
}

- (void)replacePatient:(Patient *)thePatient
{
    BOOL found = NO;
    int index = 0;
    
    while (index < [self.patientsList count] && found == NO)
    {
        if ([[self.patientsList objectAtIndex:index] mrn] == thePatient.mrn)
        {
            found = YES;
            [self.patientsList replaceObjectAtIndex:index withObject:thePatient];
        }
        index++;
    }
}
@end

