//
//  DetailViewController.m
//  MediServe
//
//  Created by Christopher Galindo on 3/11/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "DetailViewController.h"

@interface DetailViewController ()
@property (strong, nonatomic) UIPopoverController *masterPopoverController;
- (void)configureView;
@end

@implementation DetailViewController
{
    Patient *theSelectedPatient;
}

@synthesize segmentedControl = _segmentedControl;
@synthesize addCargiverButton = _addCargiverButton;
@synthesize addInterventionButton = _addInterventionButton;
@synthesize addEducationButton = _addEducationButton;
@synthesize addProblemButton = _addProblemButton;
@synthesize detailItem = _detailItem;
@synthesize detailDescriptionLabel = _detailDescriptionLabel;
@synthesize patientsName = _patientsName;
@synthesize masterPopoverController = _masterPopoverController;
@synthesize masterOneDelegate = _masterOneDelegate;
@synthesize masterTwoDelegate = _masterTwoDelegate;
@synthesize mrnLabel = _mrnLabel;
@synthesize accountLabel = _accountLabel;
@synthesize genderLabel = _genderLabel;
@synthesize birthLabel = _birthLabel;
@synthesize statusLabel = _statusLabel;
@synthesize losLabel = _losLabel;
@synthesize rehabLabel = _rehabLabel;
@synthesize firstLabel = _firstLabel;
@synthesize secondLabel = _secondLabel;
@synthesize patientStatus = _patientStatus;
@synthesize patientImageView = _patientImageView;

#pragma mark - Managing the detail item

- (void)setDetailItem:(id)newDetailItem
{
    if (_detailItem != newDetailItem) {
        _detailItem = newDetailItem;
        
        // Update the view.
        [self configureView];
    }
    
    if (self.masterPopoverController != nil) {
        [self.masterPopoverController dismissPopoverAnimated:YES];
    }        
}

- (void)configureView
{
    // Update the user interface for the detail item.
    
    if (self.detailItem) {
        self.detailDescriptionLabel.text = [self.detailItem description];
    } 
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [self configureView];
    theSelectedPatient = [[Patient alloc] init];
    
    // initializing all buttons to be hidden when detail view loads
    [self.addProblemButton setHidden:TRUE];
    [self.addInterventionButton setHidden:TRUE];
    [self.addEducationButton setHidden:TRUE];
    [self.addCargiverButton setHidden:TRUE];
    
    // setting delegate
    self.masterOneDelegate = (id<DetailViewControllerDelegate>)[[[[self.splitViewController.viewControllers objectAtIndex:0] viewControllers] objectAtIndex:0] topViewController];
    self.masterTwoDelegate = (id<DetailViewControllerDelegate>)[[[[self.splitViewController.viewControllers objectAtIndex:0] viewControllers] objectAtIndex:1] topViewController];
}

- (void)viewDidUnload
{
    [self setAddProblemButton:nil];
    [self setAddEducationButton:nil];
    [self setAddInterventionButton:nil];
    [self setAddCargiverButton:nil];
    [self setSegmentedControl:nil];
    [self setPatientsName:nil];
    [self setMrnLabel:nil];
    [self setAccountLabel:nil];
    [self setGenderLabel:nil];
    [self setBirthLabel:nil];
    [self setStatusLabel:nil];
    [self setLosLabel:nil];
    [self setRehabLabel:nil];
    [self setFirstLabel:nil];
    [self setSecondLabel:nil];
    [self setPatientImageView:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    [self.masterOneDelegate DetailViewControllerLogOut:self];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return YES;
}

#pragma mark - Split view

- (void)splitViewController:(UISplitViewController *)splitController willHideViewController:(UIViewController *)viewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)popoverController
{
    barButtonItem.title = NSLocalizedString(@"Patients", @"Patients");
    [self.navigationItem setLeftBarButtonItem:barButtonItem animated:YES];
    self.masterPopoverController = popoverController;
}

- (void)splitViewController:(UISplitViewController *)splitController willShowViewController:(UIViewController *)viewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    // Called when the view is shown again in the split view, invalidating the button and popover controller.
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    self.masterPopoverController = nil;
}

#pragma mark - IBActions
- (IBAction)getDetailTableView:(id)sender
{
    // Selecting which buttons to display depending on which segment control was selected
    NSString *selectedSeg = [self.segmentedControl titleForSegmentAtIndex:self.segmentedControl.selectedSegmentIndex];
    if ([selectedSeg isEqualToString:@"Plan of Care"])
    {
        [self.addProblemButton setHidden:FALSE];
        [self.addInterventionButton setHidden:FALSE];
        [self.addEducationButton setHidden:TRUE];
        [self.addCargiverButton setHidden:TRUE];
    }
    else if ([selectedSeg isEqualToString:@"Education"])
    {
        [self.addProblemButton setHidden:TRUE];
        [self.addInterventionButton setHidden:TRUE];
        [self.addEducationButton setHidden:FALSE];
        [self.addCargiverButton setHidden:FALSE];
    }
    else if ([selectedSeg isEqualToString:@"Information"])
    {
        [self.addProblemButton setHidden:TRUE];
        [self.addInterventionButton setHidden:TRUE];
        [self.addEducationButton setHidden:TRUE];
        [self.addCargiverButton setHidden:TRUE];
    }
}

#pragma mark - Master View Delegates
// when patient is selected in master view patient info in display in detail view
- (void)MasterViewController:(MasterViewController *)controller didSelectPatient:(Patient *)selectedPatient withStatus:(NSString *)status
{
    theSelectedPatient = selectedPatient;
    self.statusLabel.text = status;
    [self patientSelected];
}

// clearing detail view - needed for logout
- (void)MasterViewControllerClearDetailView:(MasterViewController *)controller
{
    self.patientsName.text = @"";
    self.mrnLabel.text = @"";
    self.accountLabel.text = @"";
    self.genderLabel.text = @"";
    self.birthLabel.text = @"";
    self.statusLabel.text = @"";
    self.losLabel.text = @"";
    self.rehabLabel.text = @"";
    self.firstLabel.text = @"";
    self.secondLabel.text = @"";
    
    self.patientImageView.image = nil;
}

// Closes the master view popover
- (void)masterViewControllerCloseMasterTable:(MasterViewController *)controller
{
    [self.masterPopoverController dismissPopoverAnimated:YES];
}

// same methods as Master View Delegates but these interact with master view two
#pragma mark - Master View Two Delegates
- (void)masterViewControllerTwo:(MasterViewControllerTwo *)controller didSelectPatient:(Patient *)selectedPatient
{
    theSelectedPatient = selectedPatient;
    
    // setting status label of patient selected from find patients list
    if (theSelectedPatient.isActive)
    {
        if (theSelectedPatient.isAdmitted)
        {
            self.statusLabel.text =  [[@"Admitted" stringByAppendingString:@" "] stringByAppendingString:theSelectedPatient.admitDate];
        }
        else 
        {
            self.statusLabel.text = @"Not Registered";
        }
    }
    else 
    {
        if (theSelectedPatient.isDischarged)
        {
            self.statusLabel.text = [[@"Discharged" stringByAppendingString:@" "] stringByAppendingString:theSelectedPatient.dischargeDate];
        }
        else 
        {
            self.statusLabel.text = @"Closed";
        }
    }
    
    [self patientSelected];
}

- (void)masterViewControllerTwoCloseMasterTable:(MasterViewControllerTwo *)controller
{
    [self.masterPopoverController dismissPopoverAnimated:YES];
}

#pragma mark - IBAction methods
- (IBAction)logOutButtonPressed
{
    // clears table views and logs user out of the system
    [self.masterTwoDelegate DetailViewControllerLogOut:self];
    [self.masterOneDelegate DetailViewControllerLogOut:self];
}

#pragma mark - Other Methods
// helper method to configure patient info
- (void)patientSelected;
{
    self.patientsName.text = theSelectedPatient.displayName;
    self.mrnLabel.text = theSelectedPatient.mrn;
    self.accountLabel.text = theSelectedPatient.account;
    self.genderLabel.text = theSelectedPatient.gender;
    
    UIImage *image = [UIImage imageWithData:theSelectedPatient.patientImageData];
    [self.patientImageView setImage:image];
    
    // Calculating age
    NSDate *currentDate = [[NSDate alloc] init];
    NSCalendar *calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
    NSDateComponents *components = [calendar components:NSYearCalendarUnit fromDate:theSelectedPatient.birthDate toDate:currentDate options:0];
    
    // displaying birth date and age
    NSString *tempString = [NSString stringWithFormat:@"%d", components.year];
    NSString *age = [[@"(" stringByAppendingString:tempString] stringByAppendingString:@")"];
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    NSString *birthDate = [dateFormatter stringFromDate:theSelectedPatient.birthDate];
    
    self.birthLabel.text = [[birthDate stringByAppendingString:@" "] stringByAppendingString:age];
}

@end
