//
//  MasterViewController.m
//  MediServe
//
//  Created by Christopher Galindo on 3/11/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "MasterViewController.h"
#import "DetailViewController.h"
#import "AddPatientViewController.h"
#import "DataManager.h"

@implementation MasterViewController
{
    NSMutableArray *myPatientsList;
    NSMutableArray *usersList;
    NSIndexPath *selectedIndexPath;
    NSString *alertMode;
    BOOL isLoggedin;
}

@synthesize detailViewController = _detailViewController;
@synthesize popoverViewController = _popoverViewController;
@synthesize selectedPatient = _selectedPatient;
@synthesize delegate = _delegate;
@synthesize alertView = _alertView;
@synthesize headlineTextField;

- (void)awakeFromNib
{
    self.clearsSelectionOnViewWillAppear = NO;
    self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
    [super awakeFromNib];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Do any additional setup after loading the view, typically from a nib.
    self.detailViewController = (DetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
    //[self.tableView selectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    
    // initializing instance variables
    isLoggedin = NO;
    self.selectedPatient = [[Patient alloc] init];
    myPatientsList = [[NSMutableArray alloc] initWithCapacity:1];
    self.delegate = self.detailViewController;
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    // Refreshing table every time view appears
    if (isLoggedin)
    {
        [self clearTableView];
        [self updateTableView];
    }
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
    [self.tableView deselectRowAtIndexPath:selectedIndexPath animated:NO];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return YES;
}

#pragma mark - Table View Datasource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [myPatientsList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"myPatientsCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell...
    if (isLoggedin)
    {
        UILabel *nameLabel = (UILabel *)[cell viewWithTag:100];
        
        nameLabel.text = [[myPatientsList objectAtIndex:indexPath.row] displayName];
        [self configurePatientStatus:[myPatientsList objectAtIndex:indexPath.row] andConfigureCell:cell];
    }
    return cell;
}

/*
 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
 {
 // Return NO if you do not want the specified item to be editable.
 return YES;
 }
 */


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) 
    {
        // Delete the row from the data source.
        Patient *deletedPatient = [[Patient alloc] init];
        deletedPatient = [myPatientsList objectAtIndex:indexPath.row];
        [myPatientsList removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
        
        DataManager *manager = [DataManager sharedDataManager];
        for (int index = 0; index < [deletedPatient.myClinicians count]; index++)
        {
            if ([[deletedPatient.myClinicians objectAtIndex:index] isEqualToString:manager.currentUser.userID])
            {
                [deletedPatient.myClinicians removeObjectAtIndex:index];
            }
        }
        [manager replacePatient:deletedPatient];
        
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}

/*
 // Override to support rearranging the table view.
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
 {
 }
 */

/*
 // Override to support conditional rearranging of the table view.
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
 {
 // Return NO if you do not want the item to be re-orderable.
 return YES;
 }
 */

#pragma mark - Table View Delegates
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    self.selectedPatient = [myPatientsList objectAtIndex:indexPath.row];
    selectedIndexPath = indexPath;
    
    // method call to show patient's information in the detail view
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath]; 
    UILabel *statusLabel = (UILabel *)[cell viewWithTag:101];
    [self.delegate MasterViewController:self didSelectPatient:self.selectedPatient withStatus:statusLabel.text];
    [self.delegate masterViewControllerCloseMasterTable:self];
}

#pragma mark - Login View Delegates
- (void)LoginViewController:(LoginViewController *)controller didValidateUser:(User *)userObject withUsers:(NSMutableArray *)usersArray
{
    // Setting current user property for data model
    DataManager *manager = [DataManager sharedDataManager];
    manager.currentUser = userObject;
    
    isLoggedin = YES;
    
    [self clearTableView];
    [self updateTableView];
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - StoryBoard method
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"LoginSegue"])
    {
        // setting up delegate
        LoginViewController *controller = segue.destinationViewController;
        controller.delegate = self;
        
        // passing list of users to login screen
        DataManager *manager = [DataManager sharedDataManager];
        controller.usersList = [[NSMutableArray alloc] initWithCapacity:[manager.usersList count]];
        controller.usersList = manager.usersList;
    }
    else if ([segue.identifier isEqualToString:@"AddPatientSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AddPatientViewController *viewController = (AddPatientViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        // Passing name of clinician to Add Patient View to assign patient to clinician
        DataManager *manager = [DataManager sharedDataManager];
        viewController.clinicianID = manager.currentUser.userID;
        viewController.isEditModeOn = NO;
        viewController.patientNumber = manager.patientNumber;
    }
    else if ([segue.identifier isEqualToString:@"EditPatientSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AddPatientViewController *viewController = (AddPatientViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        viewController.thePatient = self.selectedPatient;
        viewController.patientIndexPath = selectedIndexPath;
        viewController.isEditModeOn = YES;
        
        DataManager *manager = [DataManager sharedDataManager];
        viewController.clinicianID = manager.currentUser.userID;
    }
    else if ([segue.identifier isEqualToString:@"LogOutSegue"])
    {
        LoginViewController *controller = segue.destinationViewController;
        controller.delegate = self;
        
        // saving users and patients lists
        // also clearing tableview
        DataManager *manager = [DataManager sharedDataManager];
        manager.currentUser = nil;
        [manager saveUsersList];
        [manager savePatientsList];
        [self clearTableView];
        
        // passing users list to login screen
        controller.usersList = [[NSMutableArray alloc] initWithCapacity:[manager.usersList count]];
        controller.usersList = manager.usersList;
        
        // method call to clear detail view
        [self.delegate MasterViewControllerClearDetailView:self];
    }
    else if([segue.identifier isEqualToString:@"AccountsSegue"])
    {
        AccountsViewController *controller = segue.destinationViewController;
        controller.delegate = self;
        
        UIStoryboardPopoverSegue *popoverSegue;
        popoverSegue = (UIStoryboardPopoverSegue *)segue;
        self.popoverViewController = popoverSegue.popoverController;
        self.popoverViewController.delegate = self;
        
        // determines what will show in the Accounts menu
        if ([myPatientsList count] > 0)
        {
            if (self.selectedPatient.displayName == nil)
            {
                controller.accountsList = [NSArray arrayWithObject:@"New"];
            }
            else 
            {
                if (self.selectedPatient.isActive)
                {
                    if (self.selectedPatient.isAdmitted)
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Discharge", @"Merge", @"Un-do Admit", @"Headline", nil];
                    }
                    else
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Close", @"Merge", @"Admit", @"Headline", nil];
                    }
                }
                else 
                {
                    if (self.selectedPatient.isDischarged)
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Merge", @"Un-do Discharge", @"Headline", nil];
                    }
                    else
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Open", @"Merge", @"Headline", nil];
                    }
                }
            }
        }
        else 
        {
            controller.accountsList = [NSArray arrayWithObject:@"New"];
        }
    }
    else if ([segue.identifier isEqualToString:@"AdmitPatientSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AdmitPatientViewController *viewController = (AdmitPatientViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        viewController.admitPatient = self.selectedPatient;
    }
}

#pragma mark - Add Patient View delegates
- (void)addPatientViewController:(AddPatientViewController *)controller didAddPatient:(Patient *)patientObject
{
    
    [self clearTableView];
    
    // updating master patients list
    DataManager *manager = [DataManager sharedDataManager];
    [manager.patientsList addObject:patientObject];
    manager.patientNumber++;
    
    [self updateTableView];
    
    [self dismissViewControllerAnimated:YES completion:nil];
    
    // selecting patient in tableview
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:([myPatientsList count] - 1) inSection:0];
    [self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:self.tableView didSelectRowAtIndexPath:indexPath];
}

- (void)addPatientViewControllerDidCancel:(AddPatientViewController *)controller
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)addPatientViewController:(AddPatientViewController *)controller didAddUser:(User *)userObject
{
    // Adding new user to data model
    DataManager *manager = [DataManager sharedDataManager];
    [manager.usersList addObject:userObject];
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)addPatientViewController:(AddPatientViewController *)controller didEditPatient:(Patient *)patientObject withIndexPath:(NSIndexPath *)indexPath
{
    [self clearTableView];
    
    // replacing edited paitent in the data model
    DataManager *manager = [DataManager sharedDataManager];
    [manager replacePatient:patientObject];
    
    [self updateTableView];
    
    [self dismissViewControllerAnimated:YES completion:nil];
    
    // making sure patient that was edited will still be selected and showing in detail view
    // after edit was complete
    [self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:self.tableView didSelectRowAtIndexPath:indexPath];
}

#pragma mark - Detail View Delegates
- (void)DetailViewControllerLogOut:(DetailViewController *)controller
{
    [self performSegueWithIdentifier:@"LogOutSegue" sender:self];
}

#pragma mark - Other methods
// Searching master patient list for each patient that is assigned to current clinician and adding 
// that patient to my patients list
// Also populating my patients list to the table view
- (void)updateTableView
{
    DataManager *manager = [DataManager sharedDataManager];
    NSIndexPath *indexPath;
    int newIndex = 0;
    for (int index = 0; index < [manager.patientsList count]; index++)
    {
        for (int userIndex = 0; userIndex < [[[manager.patientsList objectAtIndex:index] myClinicians] count]; userIndex++)
        {
            if ([[[[manager.patientsList objectAtIndex:index] myClinicians] objectAtIndex:userIndex] isEqualToString:manager.currentUser.userID])
            {
                [myPatientsList addObject:[manager.patientsList objectAtIndex:index]];
                indexPath = [NSIndexPath indexPathForRow:newIndex inSection:0];
                NSArray *indexPaths = [NSArray arrayWithObject:indexPath];
                [self.tableView insertRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationNone];
                newIndex++; 
            }
        }
    }
}

// Clearing current table view rows
- (void)clearTableView
{
    NSIndexPath *indexPath;
    for (int index = [myPatientsList count] - 1; index >= 0; index--)
    {
        indexPath = [NSIndexPath indexPathForRow:index inSection:0];
        NSArray *removedIndexPaths = [NSArray arrayWithObject:indexPath];
        [myPatientsList removeObjectAtIndex:index];
        [self.tableView deleteRowsAtIndexPaths:removedIndexPaths withRowAnimation:UITableViewRowAnimationNone];
    }
}

// Configuring cell to display proper text and text properties for each label in cell
- (void)configurePatientStatus:(Patient *)patientForRow andConfigureCell:(UITableViewCell *)cell
{
    UILabel *notActiveLabel = (UILabel *)[cell viewWithTag:104];
    UILabel *roomLabel = (UILabel *)[cell viewWithTag:102];
    UILabel *statusLabel = (UILabel *)[cell viewWithTag:101];
    self.headlineTextField = (UITextField *)[cell viewWithTag:103];
    
    if (patientForRow.isActive)
    {
        [self.headlineTextField setHidden:NO];
        [self.headlineTextField setEnabled:NO];
        [notActiveLabel setHidden:YES];
        
        [statusLabel setTextColor:[UIColor colorWithRed:0.0 green:0.25 blue:0.50 alpha:1.0]];
        [statusLabel setHighlightedTextColor:[UIColor whiteColor]];
        
        // Determining what to display for room and bed text fields
        if ([patientForRow.admitRoom length] > 0 && [patientForRow.admitBed length] > 0)
        {
            roomLabel.text = [[patientForRow.admitRoom stringByAppendingString:@"-"] stringByAppendingString:patientForRow.admitBed];
        }
        else if ([patientForRow.admitRoom length] > 0)
        {
            roomLabel.text = [patientForRow.admitRoom stringByAppendingString:@"-?"];
        }
        else 
        {
            roomLabel.text = @"---";
        }
        
        // Determining status text field
        if (patientForRow.isAdmitted)
        {
            statusLabel.text =  [[@"Admitted" stringByAppendingString:@" "] stringByAppendingString:patientForRow.admitDate];
        }
        else 
        {
            statusLabel.text = @"Not Registered";
        }
    }
    else 
    {
        [self.headlineTextField setHidden:YES];
        [self.headlineTextField setEnabled:NO];
        [notActiveLabel setHidden:NO];
        [statusLabel setTextColor:[UIColor redColor]];
        [statusLabel setHighlightedTextColor:[UIColor redColor]];
        roomLabel.text = @"---";
        
        if (patientForRow.isDischarged)
        {
            [notActiveLabel setFont:[UIFont systemFontOfSize:32.0]];
            notActiveLabel.text = @"DISCHARGED";
            roomLabel.text = @"---";
            statusLabel.text = [[@"Discharged" stringByAppendingString:@" "] stringByAppendingString:patientForRow.dischargeDate];
        }
        else 
        {
            [notActiveLabel setFont:[UIFont systemFontOfSize:42.0]];
            notActiveLabel.text = @"CLOSED";
            roomLabel.text = @"---";
            statusLabel.text = @"Closed";
        }
    }
}

// updates patient record in data manager and makes sure patient is still selected in table view
- (void)updatePatient:(Patient *)thePatient
{
    [self clearTableView];
    
    DataManager *manager = [DataManager sharedDataManager];
    [manager replacePatient:thePatient];
    
    [self updateTableView];
    
    [self.tableView selectRowAtIndexPath:selectedIndexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:self.tableView didSelectRowAtIndexPath:selectedIndexPath];
}

#pragma mark - Accounts view delegates
- (void)AccountsViewControllerDelegate:(AccountsViewController *)controller selection:(NSString *)accountsSelection
{
    [self.popoverViewController dismissPopoverAnimated:YES];
    
    if ([accountsSelection isEqualToString:@"New"])
    {
        [self performSegueWithIdentifier:@"AddPatientSegue" sender:self];
    }
    else if ([accountsSelection isEqualToString:@"Edit"])
    {
        [self performSegueWithIdentifier:@"EditPatientSegue" sender:self];
    }
    else if ([accountsSelection isEqualToString:@"Close"])
    {
        Patient *closedPatient = self.selectedPatient;
        closedPatient.isActive = NO;
        
        [self clearTableView];
        
        [self updatePatient:closedPatient];
    }
    else if ([accountsSelection isEqualToString:@"Open"])
    {
        alertMode = @"Open";
        self.alertView = [[UIAlertView alloc] initWithTitle:@"Open Account"
                                                    message:@"Are you sure you want to open this account"
                                                   delegate:self
                                          cancelButtonTitle:@"NO"
                                          otherButtonTitles:@"YES", nil];
        [self.alertView show];
    }
    else if ([accountsSelection isEqualToString:@"Admit"])
    {
        [self performSegueWithIdentifier:@"AdmitPatientSegue" sender:self];
    }
    else if ([accountsSelection isEqualToString:@"Un-do Admit"])
    {
        alertMode = @"UndoAdmit";
        self.alertView = [[UIAlertView alloc] initWithTitle:@"Warning"
                                                    message:@"You are about to remove the  Admitted status from this patient account."
                                                   delegate:self
                                          cancelButtonTitle:@"Leave Admitted"
                                          otherButtonTitles:@"Un-do Admit action", nil];
        [self.alertView show];
    }
    else if ([accountsSelection isEqualToString:@"Discharge"])
    {
        Patient *dischargedPatient = self.selectedPatient;
        dischargedPatient.isDischarged = YES;
        dischargedPatient.isActive = NO;
        dischargedPatient.isAdmitted = NO;
        
        // Getting discharged date
        NSDate *currentDate = [[NSDate alloc] init];
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateStyle:NSDateFormatterShortStyle];
        dischargedPatient.dischargeDate = [dateFormatter stringFromDate:currentDate];
        
        [self updatePatient:dischargedPatient];
    }
    else if ([accountsSelection isEqualToString:@"Un-do Discharge"])
    {
        alertMode = @"UndoDischarge";
        self.alertView = [[UIAlertView alloc] initWithTitle:@"Warning"
                                                    message:@"You are about to move this patient account back to Admitted status"
                                                   delegate:self
                                          cancelButtonTitle:@"Leave Discharged"
                                          otherButtonTitles:@"Un-do Discharge", nil];
        [self.alertView show];
    }
    else if ([accountsSelection isEqualToString:@"Headline"])
    {
        UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:selectedIndexPath];
        self.headlineTextField = (UITextField *)[cell viewWithTag:103];
        
        [self.headlineTextField setEnabled:YES];
        [self.headlineTextField becomeFirstResponder];
    }
}

#pragma mark - Admit Patient Delegates
- (void)AdmitPatientViewController:(AdmitPatientViewController *)controller didAdmitPatient:(Patient *)admittedPatient
{
    [self dismissViewControllerAnimated:YES completion:nil];
    [self updatePatient:admittedPatient];
}

- (void)AdmitPatientViewControllerDidCancel:(AdmitPatientViewController *)controller
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Alert View Delegates
- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    if ([alertMode isEqualToString:@"Open"])
    {
        // confirms that user was to re-open account
        if (buttonIndex == 1)
        {
            Patient *openPatient = self.selectedPatient;
            openPatient.isActive = YES;
            
            [self updatePatient:openPatient];
        }
    }
    else if ([alertMode isEqualToString:@"UndoAdmit"])
    {
        // confirms that user want to un-do Admit action
        if (buttonIndex == 1)
        {
            Patient *unAdmittedPatient = self.selectedPatient;
            unAdmittedPatient.isAdmitted = NO;
            
            [self updatePatient:unAdmittedPatient];
        }
    }
    else if ([alertMode isEqualToString:@"UndoDischarge"])
    {
        // confirms that user wants to un-do discharge action
        if (buttonIndex == 1)
        {
            Patient *admittedPatient = self.selectedPatient;
            admittedPatient.isAdmitted = YES;
            admittedPatient.isActive = YES;
            admittedPatient.isDischarged = NO;
            
            [self updatePatient:admittedPatient];
        }
    }
}

#pragma mark - IBAction Methods
- (IBAction)headlineDone
{
    [self.headlineTextField resignFirstResponder];
    [self.headlineTextField setEnabled:NO];
}

@end
