//
//  MasterViewControllerTwo.m
//  MediServe
//
//  Created by Christopher Galindo on 3/11/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "MasterViewControllerTwo.h"
#import "DetailViewController.h"
#import "DataManager.h"

@implementation MasterViewControllerTwo
{
    NSMutableArray *findPatientsList;
    Patient *selectedPatient;
    NSIndexPath *selectedIndexPath;
    BOOL hasDisappeared;
    NSString *alertMode;
}

@synthesize detailViewController = _detailViewController;
@synthesize delegate = _delegate;
@synthesize popoverViewController = _popoverViewController;
@synthesize alertView = _alertView;

- (void)awakeFromNib
{
    self.clearsSelectionOnViewWillAppear = NO;
    self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
    [super awakeFromNib];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    self.detailViewController = (DetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
    self.delegate = self.detailViewController;
    
    findPatientsList = [[NSMutableArray alloc] init];
    selectedPatient = [[Patient alloc] init];
    [self updateTableView];
    hasDisappeared = NO;
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    if (hasDisappeared)
    {
        [self updateTableView];
    }
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
    
    [self clearTableView];
    hasDisappeared = YES;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return YES;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [findPatientsList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"findPatientsCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell...
    UILabel *label = (UILabel *)[cell viewWithTag:200];
    label.text = [[findPatientsList objectAtIndex:indexPath.row] displayName];
    
    return cell;
}

/*
 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
 {
 // Return NO if you do not want the specified item to be editable.
 return YES;
 }
 */

/*
 // Override to support editing the table view.
 - (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
 {
 if (editingStyle == UITableViewCellEditingStyleDelete) {
 // Delete the row from the data source.
 [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
 } else if (editingStyle == UITableViewCellEditingStyleInsert) {
 // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
 }   
 }
 */

/*
 // Override to support rearranging the table view.
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
 {
 }
 */

/*
 // Override to support conditional rearranging of the table view.
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
 {
 // Return NO if you do not want the item to be re-orderable.
 return YES;
 }
 */

#pragma mark - Table View Delegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    selectedPatient = [findPatientsList objectAtIndex:indexPath.row];
    selectedIndexPath = indexPath;
    
    [self.delegate masterViewControllerTwo:self didSelectPatient:selectedPatient];
    [self.delegate masterViewControllerTwoCloseMasterTable:self];
}

#pragma mark - Other methods
- (void)clearTableView
{
    NSIndexPath *indexPath;
    for (int index = [findPatientsList count] - 1; index >= 0; index--)
    {
        indexPath = [NSIndexPath indexPathForRow:index inSection:0];
        NSArray *removedIndexPaths = [NSArray arrayWithObject:indexPath];
        [findPatientsList removeObjectAtIndex:index];
        [self.tableView deleteRowsAtIndexPaths:removedIndexPaths withRowAnimation:UITableViewRowAnimationNone];
    }
}

- (void)updateTableView
{
    DataManager *manager = [DataManager sharedDataManager];
    NSIndexPath *indexPath;
    int newIndex = 0;
    for (int index = 0; index < [manager.patientsList count]; index++)
    {
        if ([[manager.patientsList objectAtIndex:index] isActive])
        {
            [findPatientsList addObject:[manager.patientsList objectAtIndex:index]];
            
            indexPath = [NSIndexPath indexPathForRow:newIndex inSection:0];
            NSArray *indexPaths = [NSArray arrayWithObject:indexPath];
            [self.tableView insertRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationNone];
            newIndex++; 
        }
    }
}

#pragma mark - Detail View Delegates
- (void)DetailViewControllerLogOut:(DetailViewController *)controller
{
    [self clearTableView];
}

#pragma mark - StoryBoard Method
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"MasterTwoAccountsSegue"])
    {
        AccountsViewController *controller = segue.destinationViewController;
        controller.delegate = self;
        
        UIStoryboardPopoverSegue *popoverSegue;
        popoverSegue = (UIStoryboardPopoverSegue *)segue;
        self.popoverViewController = popoverSegue.popoverController;
        self.popoverViewController.delegate = self; 
        
        // determines what will show in the Accounts menu
        if ([findPatientsList count] > 0)
        {
            if (selectedPatient.displayName == nil)
            {
                controller.accountsList = [NSArray arrayWithObject:@"New"];
            }
            else 
            {
                if (selectedPatient.isActive)
                {
                    if (selectedPatient.isAdmitted)
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Discharge", @"Merge", @"Un-do Admit", @"Assign Clinician", nil];
                    }
                    else
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Close", @"Merge", @"Admit", @"Assign Clinician", nil];
                    }
                }
                else 
                {
                    if (selectedPatient.isDischarged)
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Merge", @"Un-do Discharge", @"Assign Clinician", nil];
                    }
                    else
                    {
                        controller.accountsList = [NSArray arrayWithObjects:@"New", @"Edit", @"Open", @"Merge", @"Assign Clinician", nil];
                    }
                }
            }
        }
        else 
        {
            controller.accountsList = [NSArray arrayWithObject:@"New"];
        }
    }
    else if ([segue.identifier isEqualToString:@"MasterTwoAddPatientSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AddPatientViewController *viewController = (AddPatientViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        // Passing name of clinician to Add Patient View to assign patient to clinician
        DataManager *manager = [DataManager sharedDataManager];
        viewController.clinicianID = manager.currentUser.userID;
        viewController.isEditModeOn = NO;
        viewController.patientNumber = manager.patientNumber;
    }
    else if ([segue.identifier isEqualToString:@"MasterTwoEditPatientSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AddPatientViewController *viewController = (AddPatientViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        viewController.thePatient = selectedPatient;
        viewController.patientIndexPath = selectedIndexPath;
        viewController.isEditModeOn = YES;
        
        DataManager *manager = [DataManager sharedDataManager];
        viewController.clinicianID = manager.currentUser.userID;
    }
    else if ([segue.identifier isEqualToString:@"MasterTwoAdmitPatientSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AdmitPatientViewController *viewController = (AdmitPatientViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        viewController.admitPatient = selectedPatient;
    }
    else if ([segue.identifier isEqualToString:@"AssignClinicianSegue"])
    {
        UINavigationController *navigationController = segue.destinationViewController;
        AssignClinicianViewController *viewController = (AssignClinicianViewController *)navigationController.topViewController;
        viewController.delegate = self;
        
        DataManager *manager = [DataManager sharedDataManager];
        viewController.usersList = manager.usersList;
        viewController.currentUser = manager.currentUser;
        viewController.thePatient = selectedPatient;
    }
}

#pragma mark - Accounts view delegates
- (void)AccountsViewControllerDelegate:(AccountsViewController *)controller selection:(NSString *)accountsSelection
{
    [self.popoverViewController dismissPopoverAnimated:YES];
    
    if ([accountsSelection isEqualToString:@"New"])
    {
        [self performSegueWithIdentifier:@"MasterTwoAddPatientSegue" sender:self];
    }
    else if ([accountsSelection isEqualToString:@"Edit"])
    {
        [self performSegueWithIdentifier:@"MasterTwoEditPatientSegue" sender:self];
    }
    else if ([accountsSelection isEqualToString:@"Close"])
    {
        Patient *closedPatient = selectedPatient;
        closedPatient.isActive = NO;
     
        //[self clearTableView];
     
        [self updatePatient:closedPatient];
    }
    // this else is not needed yet. Will be needed when search function is enabled
    /*else if ([accountsSelection isEqualToString:@"Open"])
    {
        alertMode = @"Open";
        self.alertView = [[UIAlertView alloc] initWithTitle:@"Open Account" 
                                                    message:@"Are you sure you want to open this account"
                                                   delegate:self
                                          cancelButtonTitle:@"NO"
                                          otherButtonTitles:@"YES", nil];
        [self.alertView show];
    }*/
    else if ([accountsSelection isEqualToString:@"Admit"])
    {
        [self performSegueWithIdentifier:@"MasterTwoAdmitPatientSegue" sender:self];
    }
    else if ([accountsSelection isEqualToString:@"Un-do Admit"])
    {
        alertMode = @"UndoAdmit";
        self.alertView = [[UIAlertView alloc] initWithTitle:@"Warning"
                                                    message:@"You are about to remove the  Admitted status from this patient account."
                                                   delegate:self
                                          cancelButtonTitle:@"Leave Admitted"
                                          otherButtonTitles:@"Un-do Admit action", nil];
        [self.alertView show];
    }
    else if ([accountsSelection isEqualToString:@"Discharge"])
    {
        Patient *dischargedPatient = selectedPatient;
        dischargedPatient.isDischarged = YES;
        dischargedPatient.isActive = NO;
        dischargedPatient.isAdmitted = NO;
     
        // Getting discharged date
        NSDate *currentDate = [[NSDate alloc] init];
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateStyle:NSDateFormatterShortStyle];
        dischargedPatient.dischargeDate = [dateFormatter stringFromDate:currentDate];
     
        [self updatePatient:dischargedPatient];
    }
    // not needed till search functionality is added
    /*else if ([accountsSelection isEqualToString:@"Un-do Discharge"])
    {
        alertMode = @"UndoDischarge";
        self.alertView = [[UIAlertView alloc] initWithTitle:@"Warning"
                                                    message:@"You are about to move this patient account back to Admitted status"
                                                   delegate:self
                                          cancelButtonTitle:@"Leave Discharged"
                                          otherButtonTitles:@"Un-do Discharge", nil];
        [self.alertView show];
    }*/
    else if ([accountsSelection isEqualToString:@"Assign Clinician"])
    {
        [self performSegueWithIdentifier:@"AssignClinicianSegue" sender:self];
    }
}

#pragma mark - Alert View Delegates
- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    if ([alertMode isEqualToString:@"Open"])
    {
        // confirms that user was to re-open account
        if (buttonIndex == 1)
        {
            Patient *openPatient = selectedPatient;
            openPatient.isActive = YES;
            
            [self updatePatient:openPatient];
        }
    }
    else if ([alertMode isEqualToString:@"UndoAdmit"])
    {
        // confirms that user want to un-do Admit action
        if (buttonIndex == 1)
        {
            Patient *unAdmittedPatient = selectedPatient;
            unAdmittedPatient.isAdmitted = NO;
            
            [self updatePatient:unAdmittedPatient];
        }
    }
    else if ([alertMode isEqualToString:@"UndoDischarge"])
    {
        // confirms that user wants to un-do discharge action
        if (buttonIndex == 1)
        {
            Patient *admittedPatient = selectedPatient;
            admittedPatient.isAdmitted = YES;
            admittedPatient.isActive = YES;
            admittedPatient.isDischarged = NO;
            
            [self updatePatient:admittedPatient];
        }
    }
}

#pragma mark - Add Patient View Delegates
- (void)addPatientViewController:(AddPatientViewController *)controller didAddPatient:(Patient *)patientObject
{
    [self clearTableView];
    
    // updating master patients list
    DataManager *manager = [DataManager sharedDataManager];
    [manager.patientsList addObject:patientObject];
    manager.patientNumber++;
    
    [self updateTableView];
    
    [self dismissViewControllerAnimated:YES completion:nil];
    
    // selecting patient in tableview
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:([findPatientsList count] - 1) inSection:0];
    [self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:self.tableView didSelectRowAtIndexPath:indexPath];
}

- (void)addPatientViewController:(AddPatientViewController *)controller didAddUser:(User *)userObject
{
    //
}

- (void)addPatientViewController:(AddPatientViewController *)controller didEditPatient:(Patient *)patientObject withIndexPath:(NSIndexPath *)indexPath
{
    [self clearTableView];
    
    // replacing edited paitent in the data model
    DataManager *manager = [DataManager sharedDataManager];
    [manager replacePatient:patientObject];
    
    [self updateTableView];
    
    [self dismissViewControllerAnimated:YES completion:nil];
    
    // making sure patient that was edited will still be selected and showing in detail view
    // after edit was complete
    [self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:self.tableView didSelectRowAtIndexPath:indexPath];
}

- (void)addPatientViewControllerDidCancel:(AddPatientViewController *)controller
{
    //
}

#pragma mark - Other Methods
// updates patient record in data manager and makes sure patient is still selected in table view
- (void)updatePatient:(Patient *)thePatient
{
    [self clearTableView];
    
    DataManager *manager = [DataManager sharedDataManager];
    [manager replacePatient:thePatient];
    
    [self updateTableView];
    
    [self.tableView selectRowAtIndexPath:selectedIndexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];
    [self tableView:self.tableView didSelectRowAtIndexPath:selectedIndexPath];
}

#pragma mark - Admit Patient Delegates
- (void)AdmitPatientViewController:(AdmitPatientViewController *)controller didAdmitPatient:(Patient *)admittedPatient
{
    [self dismissViewControllerAnimated:YES completion:nil];
    [self updatePatient:admittedPatient];
}

- (void)AdmitPatientViewControllerDidCancel:(AdmitPatientViewController *)controller
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Assign Clinician Delegates
- (void)assignClinicianViewController:(AssignClinicianViewController *)controller assignedClinicianToPatient:(Patient *)assignedPatient
{
    [self dismissViewControllerAnimated:YES completion:nil];
    [self updatePatient:assignedPatient];
}

- (void)assignClinicianViewControllerDidCancel:(AssignClinicianViewController *)controller
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

@end